/*
    To sum the entries of an array one would use code like
       sum=0; for (i=0;i<a.length;i++) sum += a[i];
    If we did this using the methods provided by Scalar.java,
    we would create a.length+1 references of which all but the
    last would be discarded. Using this class we would
    instead make sum a ChangeableScalar and then convert it
    to a matrix. Then only one reference would be immediately
    discarded.

    The operations in this class are heavily used in  row operations.
*/

class ChangeableScalar{

    double re, im;

    ChangeableScalar(double x, double y){
        re=x; im=y;
    }

    ChangeableScalar(double x){
        re=x; im=0;
    }

    ChangeableScalar(Scalar x){
        re=x.realPart(); im=x.imaginaryPart();
    }

    Scalar toScalar(){
        return new Scalar(re,im);
    }
    void add(Scalar x){
        re += x.realPart();
        im += x.imaginaryPart();
    }
    void or(Scalar x){
        if(x.nonZero()) {re=1;im=0;}
    }
    void and(Scalar x){
        if(!x.nonZero()) {re=0;im=0;}
    }
    void max(Scalar x){
        if(x.realPart()>re) re = x.realPart();
    }
    void min(Scalar x){
        if(x.realPart()<re) re = x.realPart();
    }
    void dotp(Scalar x,Scalar y)// z = z+x*y
    {
	 re = re + (x.realPart()*y.realPart()-x.imaginaryPart()*y.imaginaryPart());
	 im = im + (x.imaginaryPart()*y.realPart()+x.realPart()*y.imaginaryPart());
    }

    void dotm(Scalar x,Scalar y)// z = z-x*y
    {
	 re = re - (x.realPart()*y.realPart()-x.imaginaryPart()*y.imaginaryPart());
	 im = im - (x.imaginaryPart()*y.realPart()+x.realPart()*y.imaginaryPart());
    }


    void dotb(Scalar x,Scalar y)// z = z+conj(x)*y
    {
	re  =  re + (x.realPart()*y.realPart()+x.imaginaryPart()*y.imaginaryPart());
	im  =  im + (x.realPart()*y.imaginaryPart()-x.imaginaryPart()*y.realPart());
    }



}