
import java.util.Vector;
import java.io.*;
import java.net.*;

class Global{


    static Vector functions; // compiled .m functions (Function)
    static MinimatApplet frontEnd;
    static Vector variables; // names of global variables (Variable)
    static Vector values; // values of global variables  (Matrix)
    /*
     INVARIANT RELATION:
     (Matrix)values.elementAt(k) is the value of the variable
     v = (Variable) variables.elementAt(k). Moreover, k = v.offset;
     This invariant relation is maintained by Function.addVariable()
     and used by Global.primeDirective below.
    */

    static final int NOTFOUND = -1;

    static void Initialize(MinimatApplet fe){
        frontEnd=fe;
        variables = new Vector();
        functions = new Vector();
        values = new Vector();
        Scalar.initialize();
        Matrix.initialize();
    }

    static void addFunction(Function func){
        functions.addElement(func);
    }

    static Function getFunction(String str) throws MinimatException{
        for(int x=0; x<functions.size();x++){
            Function func =(Function)functions.elementAt(x);
            if(str.equals(func.name)) return func;
        }

        String fileName = str+".m";
        DotMFile mFile = new DotMFile(fileName);
        if(mFile.exists()){
            TokenVector tkn = new TokenVector(mFile);
            if(tkn.isMfunction())
                return  new Function(null, tkn);// adds to table
        }
        return null;
    }//getFunction


    static String getBatch(String str) throws MinimatException{
        if(str==null) return null;
        str = str.trim();
        if(str.lastIndexOf('=')>=0) return null;
        for(int vx=0; vx<variables.size();vx++){
            Variable v=(Variable) variables.elementAt(vx);
            if(str.equals(v.name)) return null;
        }
        String fileName = str+".m";
        DotMFile mFile = new DotMFile(fileName);
        if(mFile.exists())
            return  mFile.getContents();
        else
            return null;
    }//getBatch

  /*
       The following is for direct user input. (Interpreted functions
       are invoked via Function.operate() or Function.execute().)
       Function.addVariables adds an empty matrix to Global.values
       whenever it adds a new variable to Global.variables. Therefore,
       the above invariant relation described above holds: variables
       and values corrrespond.
    */
    static void primeDirective(String text)
        throws MinimatException{
            String batch = getBatch(text);
            if(batch!=null) text=batch;
	        TokenVector tkv = new TokenVector(text);
	        //print("---\n"+text+"\n---\n");
	        //print(tkv.dump());
	        Function func = new Function(variables, tkv);
	        //print(func.dump());
	        if(tkv.mFile==null && tkv.isMfunction()){
	            // user typed in function
	            print("\n---------\n"+text+"\n----------\n");
	        }else{
                assert(func.variables==variables,"Global.java: Incorrect binding in primeDirective");
                print("\n"+MinimatApplet.outPrompt+text+"\n");
                Interpreter proc = new Interpreter(0,0,func,values);
                proc.interpret();
            }
    }


    static void print(String str){
        frontEnd.output.appendText(str);
    }

    static void print(Variable v, Matrix A){
        String str = "   "+v.name;
        if(A.nrows==1 && A.ncols==1)
          str+="  = " + A.stringRepresentation()+"\n";
        else
          str += " = [\n"+A.stringRepresentation()+"]\n";
        print(str);

    }


    static Matrix input(String msg){
         print("input not implemented");
         return new Matrix(1,1);
    }

    static void output(String str){
        print(str+"\n");
    }

    static void setFormat(Matrix A){
    }

    static void dump(Function func){
	        TokenVector tkv =  func.token;
	        print("\n\n*****"+func.name+"*****\n");
	        print(tkv.dump());
	        print(func.dump());
    }

    static void dump(){
	        print(Builtin.dump());
	        for(int x=0; x<Global.functions.size(); x++)
	            dump((Function)Global.functions.elementAt(x));
    }

    static void assert(boolean condition, String message){
    if(!condition){
    String str="\n\n*==========================================*"
               + "\n*  Minimat has detected an internal error. *"
               + "\n*  Use the following to locate the error   *"
               + "\n*  in the indicated java source file.      *\n*"
               + "\n*          " + message +"\n"
               + "\n*==========================================*\n";
    throw new IllegalArgumentException(str);
    }
    }

}// Global