class Instruction{

    int opCode; // value from CODE.java
    Object arg;
    Token token; // token corresponding to this command

    Instruction(int opCode, Object arg, Token token){
        this.opCode=opCode; this.arg=arg;
        this.token=token;
    }

    String dump(){
        String str = Code.dump(opCode);
        while (str.length()<12) str +=" ";


        if(arg instanceof Integer)
            str += "{"+ arg.toString()+"}";
        else
        if(arg instanceof Double)
            str += ((Double)arg).toString();
        else
        if(arg instanceof Variable)
            str += ((Variable)arg).name;
        else
        if(arg instanceof String)
            str += (String)arg;
        else
        if(arg instanceof Builtin)
            str += Builtin.dump((Builtin)arg);
        else
        if (arg instanceof Function)
            str += ((Function)arg).name;

        while(str.length()<25) str += " ";
        return str+"    ["+token.text+"]";
    } // dump

    boolean valid(){
        switch(opCode){
	   	case Code.LDM:
	   	case Code.STM:
	   	case Code.MSTM:

	   	case Code.LDC:
	   	case Code.STC:
	   	case Code.MSTC:

	   	case Code.LDCS:
	   	case Code.STCS:
	   	case Code.MSTCS:

	   	case Code.LDS:
	   	case Code.STS:
	   	case Code.MSTS:

	   	case Code.LDSC:
	   	case Code.STSC:
	   	case Code.MSTSC:

	   	case Code.LDSS:
	   	case Code.STSS:
	   	case Code.MSTSS: return (arg instanceof Variable);


	   	case Code.BFOR: return (arg instanceof Variable);
	   	case Code.BAT:  return (arg instanceof Integer);
	   	case Code.EFOR: return (arg == null);
	   	case Code.TST:  return (arg instanceof Integer);
	   	case Code.JMP:  return (arg instanceof Integer);
	   	case Code.BRK:  return (arg instanceof Integer);

   		case Code.OP0:
  		case Code.OP1:
  		case Code.OP2:
  		case Code.OP3:  return (arg instanceof Builtin);

	   	case Code.SICALL: return (arg instanceof Function);
	   	//case Code.SBCALL: return (arg instanceof Builtin);
	   	case Code.MICALL: return (arg instanceof Function);
	   	case Code.MBCALL: return (arg instanceof Builtin);
	   	case Code.SETI:
	   	case Code.SETO:return (arg instanceof Integer);
	   	case Code.GETI:
	   	case Code.GETO:
	   	case Code.RTN:  return (arg == null);

   		case Code.PRNT: return (arg instanceof Variable);
   		case Code.FRMT: return (arg == null);
   		case Code.VAL:  return (arg instanceof Double);
   		case Code.INPT: return (arg instanceof String);
   		case Code.OTPT: return (arg instanceof String);
   		case Code.STOP: return (arg == null);
        default:  return false;
        }
    }// valid

}// Instruction