
import java.util.*;

 class Interpreter {

    int nargsin; // number of inputs on this call
    int nargsout; // number of outputs on this call
    Function function; // function being interpreted
    Vector values;  // local variables
	Stack stack; // expression stack
	Stack result; // [result(0),...] = F()
	Stack forStack; // handles for loops
	int pc=0; // program counter



     Interpreter(int nargsin, int nargsout, Function function,
    Vector values){
        this.nargsin = nargsin;
        this.nargsout = nargsout;
        this.function = function;
        this.values = values;
        stack = new Stack();
        forStack = new Stack();
    }

     Matrix simpleResult(){
        assert(nargsout==1,"simpleResult");
        return (Matrix) values.elementAt(0);
    }

     Stack multipleResult(){
        for(int k=0; k<nargsout;k++)
            stack.push(values.elementAt(k));
        return stack;
    }

	 void interpret() throws MinimatException{
	for(;;){
	int op = ((Instruction)function.code.elementAt(pc)).opCode;
	Object arg = ((Instruction)function.code.elementAt(pc)).arg;
	switch(op){

        // load and store
  		case Code.LDM:{// load    (= push)          B=A
  		    Variable v = (Variable)arg;
  		    Matrix A = v.getValue(values);
  		    stack.push(new Matrix(A)); // makes copy
	   	} break;

  		case Code.STM:{// store  (= pop)            A=B
  		    Variable v = (Variable)arg;
  		    Matrix B = (Matrix)stack.pop();
  		    v.setValue(values,B);
	   	} break;

  		case Code.MSTM:{// multiple store            [ A      ]=F()
  		    Variable v = (Variable)arg;
  		    Matrix F = (Matrix)result.pop();
  		    v.setValue(values,F);
	   	} break;

  		case Code.LDC:{// load colon                B=A(:)
  		    Variable v = (Variable)arg;
  		    Matrix A = v.getValue(values);
  		    Matrix B = A.toColumn();
  		    stack.push(B);
	   	} break;

  		case Code.STC:{// store colon               A(:)=B
  		    Variable v = (Variable)arg;
  		    Matrix B = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    A.setColumn(B);
	   	} break;

  		case Code.MSTC:{// multiple  "               [ A(:)   ]=F()
  		    Variable v = (Variable)arg;
  		    Matrix F = (Matrix)result.pop();
  		    Matrix A = v.getValue(values);
  		    A.setColumn(F);
	   	} break;

  		case Code.LDCS:{// load colon subscript      B=A(:,j)
  		    Variable v = (Variable)arg;
  		    Matrix J = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    Matrix B = A.getColumn(J);
  		    stack.push(B);
	   	} break;

  		case Code.STCS:{// store colon subscript     A(:,j)=B
  		    Variable v = (Variable)arg;
  		    Matrix B = (Matrix)stack.pop();
  		    Matrix J = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    A.setColumn(B,J);
	   	} break;

  		case Code.MSTCS:{// multiple "                [ A(:,j) ]=F()
  		    Variable v = (Variable)arg;
  		    Matrix F = (Matrix)result.pop();
  		    Matrix J = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    A.setColumn(F,J);
	   	} break;

  		case Code.LDS:{// load subscript            B=A(i)
  		    Variable v = (Variable)arg;
  		    Matrix I = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    Matrix B = A.getRowCol(I);
  		    stack.push(B);
	   	} break;

  		case Code.STS:{// store subscript           A(i)=B
  		    Variable v = (Variable)arg;
  		    Matrix B = (Matrix)stack.pop();
  		    Matrix I = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    A.setRowCol(B,I);
	   	} break;

  		case Code.MSTS:{// multiple "                [ A(i)   ]=F()
  		    Variable v = (Variable)arg;
  		    Matrix F = (Matrix)result.pop();
  		    Matrix I = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    A.setRowCol(F,I);
	   	} break;

  		case Code.LDSC:{// load subscript colon      B=A(i,:)
  		    Variable v = (Variable)arg;
  		    Matrix I = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    Matrix B = A.getRow(I);
  		    stack.push(B);
	   	} break;

  		case Code.STSC:{// store subscript colon     A(i,:)=B
  		    Variable v = (Variable)arg;
  		    Matrix B = (Matrix)stack.pop();
  		    Matrix I = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    A.setRow(B,I);
	   	} break;

  		case Code.MSTSC:{// multiple  "               [ A(i,:) ]=F()
  		    Variable v = (Variable)arg;
  		    Matrix F = (Matrix)result.pop();
  		    Matrix I = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    A.setRow(F,I);
	   	} break;

  		case Code.LDSS:{// load subscript subscript  B=A(i,j)
  		    Variable v = (Variable)arg;
  		    Matrix J = (Matrix)stack.pop();
  		    Matrix I = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    Matrix B = A.getSubMatrix(I,J);
  		    stack.push(B);
	   	} break;

  		case Code.STSS:{// store subscript subscript A(i,j)=B
  		    Variable v = (Variable)arg;
  		    Matrix B = (Matrix)stack.pop();
  		    Matrix J = (Matrix)stack.pop();
  		    Matrix I = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    A.setSubMatrix(B,I,J);
	   	} break;

  		case Code.MSTSS:{// multiple "                [ A(i,j) ]=F()
  		    Variable v = (Variable)arg;
  		    Matrix F = (Matrix)result.pop();
  		    Matrix J = (Matrix)stack.pop();
  		    Matrix I = (Matrix)stack.pop();
  		    Matrix A = v.getValue(values);
  		    A.setSubMatrix(F,I,J);
        } break;


        // control
  		case Code.BFOR:{// begin for
  		    Variable forVariable = (Variable)arg;
  		    ForElement ele = new ForElement(forVariable, (Matrix)stack.pop());
  		    forStack.push(ele);
		    ele.forVariable.setValue(values, new Matrix(1,1));
	   	} break;

  		case Code.BAT:{// bump and test
  		    ForElement ele = (ForElement)forStack.lastElement();
  		    if(ele.count<ele.range.ncols){
  		        Matrix mi = ele.forVariable.getValue(values);
  		        mi.entry[0][0] = ele.range.entry[0][ele.count];
  		        ele.count++;
  		    }else{
  		        pc = ((Integer)arg).intValue();
  		        continue;
  		    }
	   	} break;

  		case Code.EFOR:{// end for
  		    forStack.pop();
	   	} break;

  		case Code.TST:{// test(while loops, if statements)

  		    Matrix test = (Matrix)stack.pop();
  		    if(!test.testZero()){
  		        pc = ((Integer)arg).intValue();
  		        continue;
  		    }
	   	} break;

        case Code.BRK:// break out of for or while loop
  		case Code.JMP:{// jump   "
  		    pc = ((Integer)arg).intValue();
	   	} continue;



        // function calls

        case Code.OP0:{// Builtin:()-->(*)
  		    Builtin fnc = (Builtin)arg;
   		    stack.push(fnc.operate());
   		} break;

  		case Code.OP1:{// Builtin:(*)-->(*)
  		    Builtin fnc = (Builtin)arg;
  		    Matrix A = new Matrix((Matrix) stack.pop());
   		    stack.push(fnc.operate(A));
  		} break;

  		case Code.OP2:{// Builtin:(*,*)-->(*)
  		    Builtin fnc = (Builtin)arg;
  		    Matrix B = new Matrix((Matrix) stack.pop());
  		    Matrix A = new Matrix((Matrix) stack.pop());
   		    stack.push(fnc.operate(A,B));
  		} break;

  		case Code.OP3:{// Builtin:(*,*,*)-->(*)
  		    Builtin fnc = (Builtin)arg;
  		    Matrix C = new Matrix((Matrix) stack.pop());
  		    Matrix B = new Matrix((Matrix) stack.pop());
  		    Matrix A = new Matrix((Matrix) stack.pop());
   		    stack.push(fnc.operate(A,B,C));
  		} break;

  		case Code.SICALL:{//simpleInterpretedFunctionCall
  		    Function fnc = (Function)arg;
  		    pc++;
  		    op = ((Instruction)function.code.elementAt(pc)).opCode;
  		    assert(op==Code.SETI,"SICALL+1");
  		    arg = ((Instruction)function.code.elementAt(pc)).arg;
  		    int nin = ((Integer)arg).intValue();
  		    stack.push(fnc.operate(stack,nin));
	   	} break;
  		case Code.MICALL:{//multipleInterpretedFunctionCall
  		    Function fnc = (Function)arg;
  		    pc++;
  		    op = ((Instruction)function.code.elementAt(pc)).opCode;
  		    assert(op==Code.SETI,"MICALL+1");
  		    arg = ((Instruction)function.code.elementAt(pc)).arg;
  		    int nin = ((Integer)arg).intValue();
  		    pc++;
  		    op = ((Instruction)function.code.elementAt(pc)).opCode;
  		    assert(op==Code.SETO,"MICALL+2");
  		    arg = ((Instruction)function.code.elementAt(pc)).arg;
  		    int nout = ((Integer)arg).intValue();
  		    result = fnc.execute(stack,nin,nout);
	   	} break;
  		case Code.MBCALL:{//multipleBuiltinFunctionCall
  		    Builtin fnc = (Builtin)arg;
  		    pc++;
  		    op = ((Instruction)function.code.elementAt(pc)).opCode;
  		    assert(op==Code.SETI,"MBCALL+1");
  		    arg = ((Instruction)function.code.elementAt(pc)).arg;
  		    int nin = ((Integer)arg).intValue();
  		    pc++;
  		    op = ((Instruction)function.code.elementAt(pc)).opCode;
  		    assert(op==Code.SETO,"MBCALL+2");
  		    arg = ((Instruction)function.code.elementAt(pc)).arg;
  		    int nout = ((Integer)arg).intValue();
  		    result = fnc.execute(stack,nin,nout);
	   	} break;


  		case Code.SETI:{ // set nargin
  		    assert(false, "SETI");
	   	} break;
  		case Code.SETO:{// set nargout
  		    assert(false, "SETO");
	   	} break;

  		case Code.GETI:{// get nargin
  		    stack.push(Matrix.toMatrix(nargsin));
	   	} break;

  		case Code.GETO:{// get nargout
  		    stack.push(Matrix.toMatrix(nargsout));
	   	} break;



  		//case Code.FRF:{} break;


  		case Code.PRNT:{// print a variable
  		    Variable v = (Variable)arg;
  		    Matrix A = v.getValue(values);
  		    Global.print(v,A);
   		} break;

  		case Code.FRMT:{// format statement
  		    Global.setFormat((Matrix)stack.pop());
   		} break;

// ToDo: same literal new Matrix
  		case Code.VAL:{// real literal
  		    double x = ((Double)arg).doubleValue();
  		    stack.push(Matrix.toMatrix(x));
   		} break;

  		case Code.INPT:{// display a msg, input a real
  		    stack.push(Global.input((String)arg));
   		} break;

  		case Code.OTPT:{// display a msg and start new line
  		    Global.output((String)arg);
   		} break;

  		case Code.RTN:
  		case Code.STOP: return;

        default: assert(false, "Illegal opCode");
	}// switch
	pc++;
	}//for
	}//interpret

    private void error(String message) throws MinimatException{
        throw new MinimatException(message);
    }

    private static void assert(boolean condition, String where){
        Global.assert(condition, "Interpreter.java : "+where);
    }


}// Interpreter

class ForElement{		 // for i = A. ...
		 int count=0;
		 Variable forVariable;   // i
		 Matrix range;  // value of A

		 ForElement(Variable forVariable, Matrix range){
		    this.forVariable = forVariable;
		    this.range = new Matrix(range);
		    this.range.toRow();
		}
}//ForElement