 class Matrix{
    int nrows, ncols;
    Scalar[][] entry;

     Matrix(){
        nrows = 0;
        ncols=0;
    }
    static Matrix empty;

     Matrix(int nrows, int ncols){
        this.nrows = nrows;
        this.ncols = ncols;
        entry = new Scalar[nrows][ncols];
        for(int i=0; i<nrows; i++)
            for (int j=0; j<ncols; j++)
                entry[i][j] =  Scalar.zero;
    }

     Matrix (Scalar a){
        this.nrows = 1;
        this.ncols = 1;
        entry = new Scalar[nrows][ncols];
        entry[0][0] =  a;
    }
     Matrix (Matrix A){
        this.nrows = A.nrows;
        this.ncols = A.ncols;
        entry = new Scalar[nrows][ncols];
        for(int i=0;i<nrows;i++)
            for(int j=0;j<ncols;j++)
                entry[i][j] =  A.entry[i][j];
    }

    static void initialize(){
        empty = new Matrix(0,0);
    }

    static Matrix toMatrix(double x){
        Matrix X = new Matrix(1,1);
        X.entry[0][0] =new Scalar(x);
        return X;
    }


    void rangeCheck(int i, int j)throws MinimatException{
        if(i<1||nrows<i||j<1||ncols<j) error("Subscript out of range");
    }

     Matrix toRow(){
        Matrix S = new Matrix(1,nrows*ncols);
        int k=0;
        for(int i=0; i<nrows; i++) for(int j=0; j<ncols; j++)
            S.entry[0][k++] = entry[i][j];
        return S;
    }
     Matrix toColumn(){
        Matrix S = new Matrix(nrows*ncols,1);
        int k=0;
        for(int i=0; i<nrows; i++) for(int j=0; j<ncols; j++)
            S.entry[k++][0] = entry[i][j];
        return S;
    }


     Matrix getSubMatrix(Matrix I,Matrix J)
       throws MinimatException{ //A(I,J)
        I = I.toRow(); J = J.toRow();
        Matrix S = new Matrix(I.ncols,J.ncols);
        for(int ii=0;ii<I.ncols;ii++){
            int i = I.entry[0][ii].intValueOf();
            for(int jj=0; jj<J.ncols; jj++){
                int j = J.entry[0][jj].intValueOf();
                rangeCheck(i,j);
                S.entry[ii][jj]=entry[i-1][j-1];
            }
        }
        return S;
    }

     void setSubMatrix(Matrix B, Matrix I, Matrix J)
       throws MinimatException{ //A(I,J)=B
        I = I.toRow(); J = J.toRow();
	    if (I.ncols != B.nrows || J.ncols != B.ncols)
	        error("Inconsistent assignment");
        for(int ii=0;ii<I.ncols;ii++){
            int i = I.entry[0][ii].intValueOf();
            for(int jj=0;jj<J.ncols; jj++){
                int j = J.entry[0][jj].intValueOf();
                rangeCheck(i,j);
                entry[i-1][j-1]=B.entry[ii][jj];
            }
        }
    }

     Matrix getColumn(Matrix J)throws MinimatException{ // A(:,J)
        J = J.toRow();
        Matrix S = new Matrix(nrows,J.ncols);
        for(int i=0;i<nrows;i++){
            for(int jj=0;jj<J.ncols; jj++){
                int j  = J.entry[0][jj].intValueOf();
                rangeCheck(i+1,j);
                S.entry[i][jj]=entry[i][j-1];
            }
        }
        return S;
    }

     void setColumn(Matrix B, Matrix J)throws MinimatException{ //A(:,J)=B
        J = J.toRow();
	    if (nrows!=B.nrows || J.ncols != B.ncols)
	        error("Inconsistent assignment");
        for(int i=0;i<nrows;i++){
            for(int jj=0;jj<J.ncols; jj++){
                int j  = J.entry[0][jj].intValueOf();
                rangeCheck(i+1,j);
                entry[i][j-1]=B.entry[i][jj];
            }
        }
    }

     Matrix getRow(Matrix I)throws MinimatException{//A(I,:)
        I = I.toRow();
        Matrix S = new Matrix(I.ncols,ncols);
        for(int ii=0;ii<I.ncols;ii++){
            int i = I.entry[0][ii].intValueOf();
            for(int j=0;j<ncols; j++){
                rangeCheck(i,j+1);
                S.entry[ii][j]=entry[i-1][j];
            }
        }
        return S;
    }


     void setRow(Matrix B, Matrix I)throws MinimatException{ //A(I,:)=B
        I = I.toRow();
	    if (I.ncols != B.nrows || ncols!=B.ncols)
	        error("Inconsistent assignment");
        for(int ii=0;ii<I.ncols;ii++){
            int i = I.entry[0][ii].intValueOf();
            for(int j=0;j<ncols; j++){
                rangeCheck(i,j+1);
                entry[i-1][j]=B.entry[ii][j];
            }
        }
    }

     void setColumn(Matrix B)throws MinimatException{ //A(:)=B
	    if (B.ncols*B.nrows != ncols*nrows)
	        error("Inconsistent assignement");
	    B = B.toRow();
	    int k=0;
	    for(int i=0;i<nrows;i++) for(int j=0;j<ncols;j++)
	        entry[i][j]=entry[0][k++];
    }


    Matrix getRowCol(Matrix I)throws MinimatException{// A(i)
  		    if(nrows==1) return getColumn(I);
  		    else
  		    if(ncols==1) return getRow(I);
  		    else error("Ambiguous Submatrix");
  		    return null; // to keep compiler happy
    }
    void setRowCol(Matrix B, Matrix I)throws MinimatException{// A(i)=B
  		    if(nrows==1) setColumn(B,I);
  		    else
  		    if(ncols==1) setRow(B,I);
  		    else error("Ambiguous Assignment");
    }

    boolean testZero(){
	    for(int i=0;i<nrows;i++) for(int j=0;j<ncols;j++)
		    if (!entry[i][j].nonZero()) return false;
    	return true;
    }

     static void sameSize(Matrix A, Matrix B, String opname)throws MinimatException{
        if (A.nrows!=B.nrows || A.ncols!=B.ncols)
         throw new MinimatException("Operation \""+opname+"\" requires matrices of the same size.");
    }

    private boolean noEntries(){
        return nrows==0 || ncols==0;
    }
     static Matrix range(Matrix m, Matrix n)throws MinimatException{
        if(m.noEntries() || n.noEntries()) error("empty matrix in range");
        int mm = m.entry[0][0].intValueOf();
        int nn = n.entry[0][0].intValueOf();
        if(nn<mm) return new Matrix(1,0);
        Matrix R = new Matrix(1,nn-mm+1);
        for(int i=0;i<R.ncols;i++) R.entry[0][i]= new Scalar(i+mm);
        return R;
    }
     static Matrix range(Matrix m, Matrix step, Matrix n)
        throws MinimatException{

        if(m.noEntries() || n.noEntries() || step.noEntries())
            error("empty matrix in range");
        double mr = m.entry[0][0].realPart();
        double nr = n.entry[0][0].realPart();
        double sr = step.entry[0][0].realPart();
        if((nr-mr)*sr<0) return new Matrix(1,0);
        if(!step.entry[0][0].nonZero()) error("Step size zero in range");
        Matrix R = new Matrix(1, (int)Math.round ((nr - mr) / sr)  + 1);
		for (int j=0; j<R.ncols; j++)
		   	R.entry[0][j] = new Scalar(mr + j*sr);
        return R;
    }

     static Matrix rowcat(Matrix A, Matrix B)throws MinimatException{

        if  (A.ncols != B.ncols)
    	    error("Incompatible sizes for Row catenation");
        Matrix C = new Matrix(A.nrows+B.nrows, A.ncols);
	    for (int j = 0; j < C.ncols; j++) {
    		for (int i = 0; i < A.nrows; i++)
        		C.entry[i][j]  =  A.entry[i][j] ;
      		for (int i=0; i < B.nrows; i++)
         		C.entry[A.nrows+i][j] =  B.entry[i][j] ;
    	}
    	return C;
    }
     static Matrix colcat(Matrix A, Matrix B)throws MinimatException{
    	if  (A.nrows != B.nrows)
    	    error("Incompatible sizes for Column catenation");
    	Matrix C = new Matrix(A.nrows, A.ncols + B.ncols);
    	for (int i = 0; i <  C.nrows; i++) {
      		for (int j = 0; j < A.ncols; j++)
         		C.entry[i][j] =  A.entry[i][j];
      		for (int j = 0; j < B.ncols; j++)
         		C.entry[i][A.ncols+j] =  B.entry[i][j];
    	}
    	return C;
    }

    static Matrix scalarMultiply(Scalar c,Matrix A){
	    Matrix B = new Matrix(A.nrows, A.ncols);
	    for (int i = 0; i < B.nrows; i++)
    	    for (int j = 0; j < B.ncols; j++)
        	    B.entry[i][j] = Scalar.multiply(c, A.entry[i][j]);
 	    return B;
    }
    static Matrix matrixMultiply(Matrix A, Matrix B){
	    assert(A.ncols==B.nrows,"matrixMultiply");
	    Matrix C = new Matrix(A.nrows, B.ncols);
        for (int i=0; i<C.nrows; i++)
	        for (int k=0;  k<C.ncols; k++){
		        ChangeableScalar s  = new ChangeableScalar(Scalar.zero);
		        for(int j =0; j < A.ncols; j++)// s = s + A[i,j]*B[j,k]
			        s.dotp(A.entry[i][j],  B.entry[j][k]);
			    C.entry[i][k] = s.toScalar();
		    }
	    return C;
    }
    static Matrix multiply(Matrix A, Matrix B)throws MinimatException{
        if (A.nrows==1 && A.ncols==1)
      		return scalarMultiply(A.entry[0][0], B);
	    else
	    if (B.nrows==1 && B.ncols==1)
      		return scalarMultiply(B.entry[0][0], A);
	    else
    	if (A.ncols == B.nrows)
        	return matrixMultiply(A,B);
      	else{
      	    error("Incompatible sizes for matrix multiplication");
      	    return null;
      	}
    }
     static Matrix divide(Matrix A, Matrix B)throws MinimatException{
	    if (B.nrows!=1 || B.ncols!=1)
	        error("Only division by a scalar is defined");
	    Scalar c = Scalar.divide(Scalar.one, B.entry[0][0]);
	    return scalarMultiply(c,A);
    }

     static Matrix power(Matrix A, Matrix N)throws MinimatException{
	    if (A.nrows==1 && A.ncols==1 && N.nrows==1 && N.ncols==1)
		    return new Matrix(Scalar.power(A.entry[0][0], N.entry[0][0]));
		if(A.nrows!=A.ncols || N.nrows!=1 || N.ncols!=1)
		    error("Only integer power of square matrix is defined");
		if(!Scalar.integral(N.entry[0][0]))
		    error("Exponent must be an integer");
	    int e = N.entry[0][0].intValueOf();
	    Matrix V;
	    if (e < 0) {
		    V = inverse(A);
		    e = -e;
	    }else
		    V = new Matrix(A);
		Matrix Z = identity(V.nrows);
	    for (;;){
		    if ( e % 2 == 1) Z = matrixMultiply(Z,V);
	  	    e  = e / 2;
		    if (e == 0) return Z;
	  	    V = matrixMultiply(V,V);
	    }
    }

    static Matrix inverse(Matrix A)throws MinimatException{
        if(A.nrows!=A.ncols) error("Can only invert a square matrix");
        A = new Matrix(A); // Make copy
        int n = A.ncols;
        Matrix B = identity(n);
        for(int r=0; r<n; r++){
            int k=0;
            double z = -1;
            for(int i=r; i<n; i++) {
                double zz = Scalar.abs1(A.entry[i][r]);
                if(zz>z){ z=zz; k=i;}
            }
            if(z<0.00000000000001) error("Matrix not invertible");
            A.swapRow(k,r); B.swapRow(k,r);
            Scalar c = Scalar.divide(Scalar.one,A.entry[r][r]);
            A.scaleRow(r,c);  B.scaleRow(r,c);
            for(int i=0; i<n; i++) if(i!=r){
                c = Scalar.subtract(A.entry[i][r]);
                A.shearRow(i,r,c); B.shearRow(i,r,c);
            }
        }
        return B;
    }
    static Matrix extractDiagonal(Matrix A,int k){
      	//(i,j) on the k'th diagonal <==> i-j=k
      	//len = #{ (i,j): 1 <= i <= A.nrows, 1 <= j <= A.ncols,  i-j=k }
	    int k0 ,len;
	    if (k>=0) {
		    k0 = k;
		    if (A.nrows + k <= A.ncols)
			    len = A.nrows;
		    else
			    len = A.nrows - k;
	    } else{
          	k0 = -k;
          	if ( A.nrows + k <= A.ncols )
          		len = A.nrows + k;
          	else
          		len = A.ncols;
	    }
	    if (len <= 0) return new Matrix(1,0);
    	else{
         	Matrix R = new Matrix(1,len);
         	if (k >= 0)
           		for(int j = 0; j < len; j++)
           		 	R.entry[0][j] = A.entry[j][j+k0];
		    else // k<0
      			for(int j = 0; j < len; j++)
           		 	R.entry[0][j]=  A.entry[j+k0][j];
	        return R;
	    }
    }


    static Matrix makeDiagonal(Matrix R, int k){
	    int k0 = k<0 ? -k : k;

	    if (R.ncols == 1) R= R.toRow();
	    Matrix A = new Matrix(R.ncols+k0, R.ncols+k0);
	    if (k >= 0)
		for(int i = 0; i <  R.ncols; i++)
			 A.entry[i][i+k0] =  R.entry[0][i];
    	else
      		for(int i = 0; i < R.ncols; i++)
		  	 A.entry[i+k0][i] =  R.entry[0][i];
	    return A;
    }

     static Matrix diag(Matrix A){
        if (A.nrows>1 && A.ncols>1)
		    return extractDiagonal(A, 0);
    	else
        	return makeDiagonal(A, 0);
    }


     static Matrix diag(Matrix X, Matrix Y)throws MinimatException{
	    if (X.nrows==1 && X.ncols==1 && Y.nrows==1 && Y.ncols==1){
		    Matrix A = new Matrix (2,2);
		    A.entry[0][0] =  X.entry[0][0];
	        A.entry[1][1] =  Y.entry[0][0];
		    return A;
	    }
		if(Y.nrows!=1 || Y.ncols!=1 || !Scalar.integral(Y.entry[0][0]))
		    error("Integer required in diag");
	    int k = Y.entry[0][0].intValueOf();
        if (X.nrows>1 && X.ncols>1)
            return extractDiagonal(X,k);
        else
            return makeDiagonal(X,k);
    }




     static Matrix commutator(Matrix A, Matrix B)throws MinimatException{
        if(A.nrows!=A.ncols || B.nrows!=B.ncols)
            error("Square matrices required");
        if(A.nrows!=B.nrows)
            error("Matrices must be same size");
        Matrix AB = matrixMultiply(A,B);
        Matrix BA = matrixMultiply(B,A);
        for(int i=0;i<A.nrows;i++)
            for(int j=0;j<A.nrows;i++)
                AB.entry[i][j]=
                    Scalar.subtract(AB.entry[i][j],BA.entry[i][j]);

        return AB;
    }

    static Matrix xpose(boolean flag, Matrix A){
	    Matrix B  = new Matrix(A.ncols, A.nrows);
    	for (int i = 0; i <  A.nrows; i++)
    	for (int j = 0; j <  A.ncols; j++)
    		if (flag)
    			B.entry[j][i] =  A.entry[i][j];
    		else
    			B.entry[j][i] = Scalar.conj(A.entry[i][j]);
    	return B;
    }
     static Matrix conjpose(Matrix A){
        return  xpose(false,A);
    }
     static Matrix transpose(Matrix A){
        return  xpose(true,A);
    }

     static Matrix zeros(){
        return new Matrix(1,1);
    }
     static Matrix zeros(Matrix n)throws MinimatException{
        if(n.noEntries()) error("Empty matrix in zeros()");
        int nn = n.entry[0][0].intValueOf();
        return new Matrix(1,nn);
    }
     static Matrix zeros(Matrix m, Matrix n)throws MinimatException{
        if(m.noEntries()|| n.noEntries()) error("Empty matrix in zeros()");
        int mm = m.entry[0][0].intValueOf();
        int nn = n.entry[0][0].intValueOf();
        return new Matrix(mm,nn);
    }

     static Matrix identity(int n){
        Matrix I = new Matrix(n,n);
        for(int i=0;i<I.ncols;i++)
            I.entry[i][i]=Scalar.one;
        return I;
     }
     static Matrix eye(Matrix n)throws MinimatException{
        if(n.noEntries()) error("Empty matrix in eye()");
        int nn = n.entry[0][0].intValueOf();
        return identity(nn);
    }
     static Matrix ones(Matrix n)throws MinimatException{
        if(n.noEntries()) error("Empty matrix in ones()");
        Matrix I = zeros(n);
            for(int j=0;j<I.ncols;j++)
                I.entry[0][j]=Scalar.one;
        return I;
    }
     static Matrix ones(Matrix m, Matrix n)throws MinimatException{
        if(m.noEntries()|| n.noEntries()) error("Empty matrix in ones()");
        Matrix I = zeros(m,n);
        for(int i=0;i<I.nrows;i++)
            for(int j=0;j<I.ncols;j++)
                I.entry[i][j]=Scalar.one;
        return I;
    }

     static Matrix rand(){
        Matrix R = zeros();
        R.entry[0][0]=new Scalar(Math.random());
        return R;
    }
     static Matrix rand(Matrix n)throws MinimatException{
        if(n.noEntries()) error("Empty matrix in rand");
        Matrix R = zeros(n);
        for(int j=0;j<R.ncols;j++)
            R.entry[0][j]=new Scalar(Math.random());
        return R;
    }
     static Matrix rand(Matrix m, Matrix n)throws MinimatException{
        if(m.noEntries()|| n.noEntries()) error("Empty matrix in rand");
        Matrix R = zeros(m,n);
        for(int i=0;i<R.nrows;i++)
            for(int j=0;j<R.ncols;j++)
                R.entry[i][j]=new Scalar(Math.random());
        return R;
    }

/*
The following prototype performs the operation func
on each column of the matrix A and returns the row
of results.

    static Matrix columnOp(Matrix A){
        A = new Matrix(A);// make copy
        if(A.nrows==1) A = A.toColumn();
	    Matrix R = new Matrix(1, A.ncols);
	    for(int j = 0; j <  A.ncols; j++){
		    ChangeableScalar s = init;
		    for(int i = 0; i < A.nrows; i++)
	   		    s.func(A.entry[i][j]);
		    R.entry[0][j] = s.toScalar();
	    }
	    return R;
    }
*/
     static Matrix sum(Matrix A){
        A = new Matrix(A);// make copy
        if(A.nrows==1) A = A.toColumn();
	    Matrix R = new Matrix(1, A.ncols);
	    for(int j = 0; j <  A.ncols; j++){
		    ChangeableScalar s = new ChangeableScalar(Scalar.zero);
		    for(int i = 0; i < A.nrows; i++)
	   		    s.add(A.entry[i][j]);
		    R.entry[0][j] = s.toScalar();
	    }
	    return R;
    }

     static Matrix any(Matrix A){
        A = new Matrix(A);// make copy
        if(A.nrows==1) A = A.toColumn();
	    Matrix R = new Matrix(1, A.ncols);
	    for(int j = 0; j <  A.ncols; j++){
		    ChangeableScalar s = new ChangeableScalar(Scalar.zero);
		    for(int i = 0; i < A.nrows; i++)
	   		    s.or(A.entry[i][j]);
		    R.entry[0][j] = s.toScalar();
	    }
	    return R;
    }
     static Matrix all(Matrix A){
        A = new Matrix(A);// make copy
        if(A.nrows==1) A = A.toColumn();
	    Matrix R = new Matrix(1, A.ncols);
	    for(int j = 0; j <  A.ncols; j++){
		    ChangeableScalar s = new ChangeableScalar(Scalar.one);
		    for(int i = 0; i < A.nrows; i++)
	   		    s.and(A.entry[i][j]);
		    A.entry[0][j] = s.toScalar();
	    }
	    return R;
    }

     static Matrix min(Matrix A){
        A = new Matrix(A);// make copy
        if(A.nrows==1) A = A.toColumn();
	    Matrix R = new Matrix(1, A.ncols);
	    for(int j = 0; j <  A.ncols; j++){
		    ChangeableScalar s = new ChangeableScalar(Scalar.plusInfinity);
		    for(int i = 0; i < A.nrows; i++)
	   		    s.min(A.entry[i][j]);
		    R.entry[0][j] = s.toScalar();
	    }
	    return R;
    }
     static Matrix max(Matrix A){
        A = new Matrix(A);// make copy
        if(A.nrows==1) A = A.toColumn();
	    Matrix R = new Matrix(1, A.ncols);
	    for (int j = 0; j <  A.ncols; j++){
		    ChangeableScalar s = new ChangeableScalar(Scalar.minusInfinity);
		    for (int i = 0; i < A.nrows; i++)
	   		    s.max(A.entry[i][j]);
		    R.entry[0][j] = s.toScalar();
	    }
	    return R;
    }



/*
    To do [R W] = func(A) (where func = max or min)
    do   R = func(A), W = columnMatch(A,R)
    Then W holds the index of the entry where max or min
    is achieved.

*/

    static Matrix columnMatch(Matrix A, Matrix R)throws MinimatException{
        assert(R.nrows==1 && R.ncols==A.ncols,"columnMatch");
	    Matrix W = new Matrix(1, A.ncols);
	    for (int j = 0; j <  A.ncols; j++)
	        for (int i = 0; i <  A.nrows; i++)
		        if (A.entry[i][j].realPart()==R.entry[0][j].realPart())
	   		        W.entry[0][j]  = new Scalar(i+1);
	    return W;
    }

/*************************** Elementry operations *****************/


void scaleRow(int p, Scalar c){//A(p,:) = c*A(p,:)
	for (int j = 0; j < ncols; j++)
		entry[p][j] = Scalar.multiply(c,entry[p][j]);
}

void scaleCol(int p, Scalar c){// A(:,p) = c*A(:,p)
	for (int i = 0; i<nrows; i++)
		entry[i][p] = Scalar.multiply(c,entry[i][p]);
}


void swapRow(int p, int q){// A(p,:) <==> A(q,:)
	for(int j=0; j<ncols; j++){
        Scalar tmp = entry[p][j];
		entry[p][j] = entry[q][j];
 		entry[q][j] = tmp;
	}
}

void swapCol(int p, int q){//A(:,p) <==> A(:,q)
	for(int i=0; i<nrows; i++){
		Scalar tmp = entry[i][p];
		entry[i][p] = entry[i][q];
 		entry[i][q] = tmp;
	}
}


void shearRow(int p,int q, Scalar c){//A(p,:) = A(p,:) + c*A(q,:)
	for (int j=0; j<ncols; j++)
	    entry[p][j] = Scalar.add(entry[p][j], Scalar.multiply(c,entry[q][j]));
}


void shearCol(int p,int q, Scalar c){// A(:,p) = A(:,p) + c*A(:,q)
	for (int i=0; i<nrows; i++)
	    entry[i][p] = Scalar.add(entry[i][p], Scalar.multiply(c,entry[i][q]));
}


/***************** Formating ******************************/

String stringRepresentation(){
    String str = "";
    for(int i=0; i<nrows; i++){
        for(int j=0; j<ncols; j++)
            str += "  "+ entry[i][j].stringRepresentation();
        str+="\n";
    }
    return str;
}



    private static void error(String message) throws MinimatException{
        throw new MinimatException(message);
    }
    private static void assert(boolean condition, String message){
        Global.assert(condition, "Matrix: "+message);
    }

}// Matrix
