/**
    We want to think of Scalar as a primitive type:
    we don't want one part of the program to change a
    Scalar refered to by a different variable in another
    part of the program. The way to assure this is to
    make sure that a Scalar cannot be changed after it
    is created. Thus all  fields  are private and
    the only place a field of a Scalar is changed is in
    this class, and then only when the ambient object is
    being created. The outside world may have many
    references to the same object, but that object is
    immutable during its existence.

*/
 class Scalar{

    private double re, im;

    private static final double TOL = 0.0000000000001;
    // Used to decide if double is an integer

    Scalar(double x, double y){
        re=x; im=y;
    }

    Scalar(double x){
        re=x; im=0;
    }





    static Scalar zero, one, minusOne, imaginaryUnit, pi,
                plusInfinity,minusInfinity;


    static Scalar toScalar(boolean b){
        return b ? one : zero;
    }


    static void initialize(){
        zero = new Scalar(0,0);
        one = new Scalar(1,0);
        minusOne = new Scalar(-1,0);
        imaginaryUnit = new Scalar(0,1);
        pi = new Scalar(Math.PI,0);
        plusInfinity = new Scalar(Double.POSITIVE_INFINITY,0);
        minusInfinity = new Scalar(Double.NEGATIVE_INFINITY,0);
    }

    // conversion methods


    int intValueOf(){
        return (int)re;
    }
    boolean nonZero(){
        return abs1(this)>=TOL;
    }

    double realPart(){
            return re;
    }

    double imaginaryPart(){
            return im;
    }

    String double2String(double x){
        String str;
        if(Math.abs(x)>0.0000001|| x==0)
            str = Double.toString(x);
        else
            str = "(0)";
        return str;
    }
    String stringRepresentation(){
        String  str = double2String(re);
        if( im>0.0000001){
            str += "+"+double2String(im)+"*i";
            while(str.length()<19) str +=" ";
        }else
        if(-im>0.0000001){
            str += "-"+double2String(-im)+"*i";
            while(str.length()<19) str +=" ";
        } else
            while(str.length()<9) str +=" ";
        return str;
    }

     static Scalar add(Scalar x, Scalar y){
        return new Scalar(x.re+y.re, x.im+y.im);
    }
     static Scalar subtract(Scalar x, Scalar y){
        return new Scalar(x.re-y.re, x.im-y.im);
    }
     static Scalar subtract(Scalar x){
        return new Scalar(-x.re, -x.im);
    }
     static Scalar multiply(Scalar x, Scalar y){
        return new Scalar(x.re*y.re-x.im*y.im, x.re*y.im+x.im*y.re);
    }

    static Scalar divide(Scalar x, Scalar y){
	    if (Math.abs(y.re) > Math.abs(y.im)){
		    double h = y.im/y.re;
		    y.re = h*y.im+y.re;
		    return new Scalar((x.re+h*x.im)/y.re,(x.im-h*x.re)/y.re);
	    }else{
		    double h = y.re/y.im;
		    y.im  = h*y.re+y.im;
		    return new Scalar((h*x.re+x.im)/y.im, (h*x.im-x.re)/y.im);
	    }
    }

    static boolean integral(Scalar z){
        return  Math.abs(Math.rint(z.re)-z.re)<TOL && Math.abs(z.im)<TOL;
    }

    static Scalar power(Scalar x, Scalar y){
	    int m,n;
	    Scalar z = new Scalar(1,0);
	    if (integral(y)) {
	        m = (int)Math.round(y.re);
		    if (m>0)
			    n = m;
		    else{
			    x = divide(z,x);
			    n = -m;
		    }
		    for(;;){
			    if (n % 2 == 1) z = multiply(z,x);
			    n = n / 2;
			    if (n==0) break;
			    x = multiply(x,x);
		    }
	    }
	    else if ( abs1(x) <TOL )
	        return zero;
	    else{
		    x = Scalar.log(x);
		    z.re = x.re*y.re - x.im*y.im;
		    z.im = x.re*y.im + x.im*y.re;
		    z = Scalar.exp(z);
	    }
	    return z;
    }

    static Scalar gtr(Scalar x, Scalar y){
        return x.re>y.re ? one : zero;
    }
    static Scalar geq(Scalar x, Scalar y){
        return x.re>=y.re ? one : zero;
    }
    static Scalar equ(Scalar x, Scalar y){
        return x.re==y.re && x.im==y.im ? one : zero;
    }
    static Scalar neq(Scalar x, Scalar y){
        return x.re!=y.re || x.im!=y.im ? one : zero;
    }
    static Scalar les(Scalar x, Scalar y){
        return x.re<y.re ? one : zero;
    }
    static Scalar leq(Scalar x, Scalar y){
        return x.re<=y.re ? one : zero;
    }
    static Scalar and(Scalar x, Scalar y){
     return (Math.abs(x.re)<TOL || Math.abs(y.re)<TOL) ? zero : one;
    }
    static Scalar or(Scalar x, Scalar y){
     return (Math.abs(x.re)<TOL && Math.abs(y.re)<TOL) ? zero : one;
    }
    static Scalar not(Scalar x){
        return Math.abs(x.re)<TOL ? one: zero; ;
    }

    static Scalar  real(Scalar x) {
        return new Scalar(x.re,0);
    }
    static Scalar  imag(Scalar x) {
        return new Scalar(0,x.im);
    }
    static Scalar  conj(Scalar x) {
        return new Scalar(x.re,-x.im);
    }


    static Scalar  round(Scalar x) {
        return new Scalar(Math.rint(x.re),Math.rint(x.im));
    }
    static Scalar  fix(Scalar x) {
        return new Scalar(Math.floor(x.re),Math.floor(x.im));
    }

    static double abs1(Scalar x){
        return Math.abs(x.re) + Math.abs(x.im);
    }

    static double abs2(Scalar x){
        return Math.sqrt(x.re*x.re + x.im*x.im);
    }

    static Scalar  abs(Scalar x) {
        return new Scalar(abs2(x),0);
    }

    static Scalar  sign(Scalar x) {
        return x.re>0 ? one : x.re<0 ? minusOne : zero;
    }
    static Scalar  sqrt(Scalar x) {
	    double h = Math.sqrt((Math.abs(x.re)+abs2(x))/2.0);
	    Scalar y = new Scalar(0,0);
	    if (h != 0.0) y.im = x.im/(2.0*h);
	    else          y.im = 0.0;

	    if (x.re >= 0.0)
	        y.re = h;
	    else if (x.im >= 0.0) {
		    y.re = y.im;
		    y.im = h;
	    }else{
		    y.re = -y.im;
		    y.im = -h;
	    }
	    return y;
	}
    static Scalar  exp(Scalar x) {
	    double r = Math.exp(x.re);
	    return new Scalar(r*Math.cos(x.im),r*Math.sin(x.im));
    }
    static Scalar  log(Scalar x) {// y=log(abs2(x))+i*atan2(x.im,x.re)
	    double r = abs2(x);
	    return new Scalar(Math.log(r),Math.atan2(x.im,x.re));
    }

    static Scalar pi(){
        return new Scalar(Math.PI);
    }

    static Scalar  sin(Scalar x) {
	    double ee, ff;
	    ee = Math.exp(x.im);
	    ff = 1.0/ee;
	    return new Scalar(Math.sin(x.re)*(ee+ff)/2.0,
	                      Math.cos(x.re)*(ee-ff)/2.0);
    }
    static Scalar  cos(Scalar x) {
	    double ee, ff;
	    ee = Math.exp(x.im);
	    ff = 1.0/ee;
	    return new Scalar(Math.cos(x.re)*(ee+ff)/2.0,
	                     -Math.sin(x.re)*(ee-ff)/2.0);
    }
    static Scalar  tan(Scalar x) {
	    Scalar c = cos(x);
	    Scalar s = sin(x);
	    return divide(s,c);
    }
    static Scalar  atan(Scalar x) { //y= (i/2)*log((i+x)/(i-x))
        Scalar u = new Scalar(x.re,1+x.im);
        Scalar v = new Scalar(-x.re,1-x.im);
	    u = divide(u,v);
	    u = log(u);
	    return new Scalar( -u.im/2.0, u.re/2.0);
    }
    static Scalar  atan2(Scalar x, Scalar y) {
        return new Scalar(Math.atan2(x.re,y.re),0);
    }
    static Scalar  asin(Scalar x) { //y= -i*log(i*x+csqrt(1-x*x))
        Scalar u = new Scalar(1.0-x.re*x.re+x.im*x.im -2.0*x.re*x.im);
	    Scalar v = sqrt(u);
	    u.re = -x.im+v.re;
	    u.im =  x.re+v.im;
	    v = log(u);
	    return new Scalar(v.im,-v.re);
    }
    static Scalar  acos(Scalar x) { //y= -i*log(x+i*csqrt(1-x*x)
        Scalar u = new Scalar(1.0-x.re*x.re+x.im*x.im,-2.0*x.re*x.im);
	    Scalar v = sqrt(u);
	    u.re = x.re-v.im;
	    u.im = x.im+v.re;
	    v = log(u);
	    return new Scalar(v.im,-v.re);
    }


}//Scalar